/*
 * Copyright (C) 2016 - 2017 Radek Hranicky, Vojtech Vecera
 * 
 * Permission is hereby granted, free of charge, to any person obtaining a copy 
 * of this software and associated documentation files (the "Software"), to deal 
 * in the Software without restriction, including without limitation the rights 
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell 
 * copies of the Software, and to permit persons to whom the Software is 
 * furnished to do so, subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included in 
 * all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR 
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, 
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE 
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER 
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, 
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE 
 * SOFTWARE.
 */

#ifndef PARSER_H
#define	PARSER_H

#include <iostream>
#include <string>
#include <fstream>
#include <cstdio>
#include <vector>
#include <chrono>
#include <iomanip>
#include <ctime>

#include "control.h"
#include "base64.h"

#define CHARSET_XML_PATH "charset.xml"
#define DICTIONARY_NAME "dict"

/**
 * @brief   Initializes values of taskParams to default.
 */
void params_init();

/**
 * @brief   Saves parameter values from the input file to the taskParams
 *	    structure.
 * @param inputFileName
 */
void parse_task_params(std::string inputFileName, TTaskParams& taskParams);

/**
 * @brief   
 * @param programOutput
 * @return  
 */
std::string look_for_password(std::string programOutput);

/**
 * @brief   Safely reads line char by char from std::streambuf.   
 * @param is
 * @param t
 * @return is 
 */
std::istream& safe_get_line(std::istream& is, std::string& t);

 /**
 * @brief   Parses line of the 'in' file in the format introduced in January
 *	    2017 by Lukas Zobal.
 * @param message
 * @param part1
 * @param part2
 */
bool parse_message_new_config(std::string message, std::string& part1, std::string& part2);

void hashcat_parser (char buf, std::string& outline, std::vector<std::string>& status);

#endif /* PARSER_H */
