/*
 * Main part of the runner with all the paths and threading 
 * 
 * Copyright (C) 2017 Vojtech Vecera
 * 
 * Permission is hereby granted, free of charge, to any person obtaining a copy 
 * of this software and associated documentation files (the "Software"), to deal 
 * in the Software without restriction, including without limitation the r  ghts 
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell 
 * copies of the Software, and to permit persons to whom the Software is 
 * furnished to do so, subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included in 
 * all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR 
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, 
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE 
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER 
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, 
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE 
 * SOFTWARE.
 */

#include "main.h"

#define LINUX32_EXEC "hashcat32.bin"
#define LINUX64_EXEC "hashcat64.bin"
#define WINDOWS32_EXEC "hashcat32.exe"
#define WINDOWS64_EXEC "hashcat64.exe"
#define LINUX_PATH_TO_HASHCAT "./"
#define LOCAL_CONFIG_NAME "local.conf"
#define LOCAL_CONFIG_PATH_WIN "C:\\fitcrack\\"
#define LOCAL_CONFIG_PATH_LINUX "/var/lib/fitcrack/"
#define KERNELS "hashcatkernels1.zip"

Control c;

int main(int argc, char** argv) {
    int retval = 1;
    char buf[256];
    std::string hashFile;

    std::string kernels("hashcatkernels1.zip");
    zip_extract(kernels.c_str());

    if(!boinc_is_standalone()) {
	BOINC_OPTIONS options;

	boinc_options_defaults(options);
	options.multi_thread = true;    // if your app's main process uses multiple threads
	options.multi_process = true;   // if your app uses multiple processes
	options.normal_thread_priority = true;

	retval = boinc_init_options(&options);


	//std::cerr << "Running with BOINC" << std::endl;
//	std::cerr << __FILE__ << " " << __LINE__ << " boinc_init()" << std::endl;
	parse_task_params(resolve_input_file("config"), c.taskParams);
    } else {
	std::cout << "Running without BOINC" << std::endl;
        parse_task_params("config", c.taskParams);
    }
    /**
     * @brief   On boinc_init error print error on STDERR
     */
    if (retval) {
	std::cerr << boinc_msg_prefix(buf, sizeof(buf)) <<" boinc_init returned " << retval << std::endl;
	boinc_finish(retval);
    }

    std::thread t (launch_hashcat);
    // wait for thread to end 
    t.join();
    if(!boinc_is_standalone()) {
	c.create_output_file(!boinc_is_standalone());
	boinc_fraction_done(1);
	boinc_finish(0);
    } else {
	std::cout << "ExitCode: " << c.exitCode << " Error: " << c.errorText << std::endl;
	std::cout << "Password: " << c.password << std::endl;
    }

    return 0;
}

inline bool file_exists (const std::string& name) {
#ifdef __linux__
    struct stat buffer;
    return (stat (name.c_str(), &buffer) == 0);
#elif _WIN32
    struct _stat buffer;
    return (_stat (name.c_str(), &buffer) == 0);
#else
    return false;
#endif
}

int zip_extract(const char *archive) {
    struct zip *za;
    struct zip_file *zf;
    struct zip_stat sb;
    char buf[100];
    int err;
    int i, len;
    int fd;
    uint64_t sum;

    if ((za = zip_open(archive, 0, &err)) == NULL) {
	return 1;
    }

    char * pch;
    char directory[256];
    int index;
#ifdef _WIN32
    GetCurrentDirectory(256, directory);
    std::string rootPath(directory);
    std::string absPath;
#endif
    memset(directory, 0, 256);
    for (i = 0; i < zip_get_num_entries(za, 0); i++) {
	if (zip_stat_index(za, i, 0, &sb) == 0) {

	    /**
	     * @brief Creates  directory tree from ZIP
	     */
	    pch = strchr((char *)sb.name, '/');
	    while(pch != NULL)
	    {
		index = pch - sb.name + 1;
		memcpy( directory, sb.name, index);
#ifdef _WIN32
		absPath = rootPath;
		absPath += "\\";
		absPath += directory;
		char& last = absPath.back();
		if (last == '/') {
		    absPath.pop_back();
		    absPath.push_back('\\');
		}
#endif
		directory[index] = '\0';
#if defined (_WIN32)
		safe_create_dir(absPath.c_str());
#elif defined (__linux__)
		safe_create_dir((const char*)directory);
#endif 
		pch=strchr(pch + 1, '/');
	    }

	    len = strlen(sb.name);
#if defined (_WIN32)
	    absPath = rootPath;
	    absPath += "\\";
	    absPath += sb.name;
#endif 
	    if (sb.name[len - 1] == '/') {
#if defined (_WIN32)
		absPath += "\\";
		safe_create_dir(absPath.c_str());
#elif defined (__linux__)
		safe_create_dir(sb.name);
#endif 
	    } else {
		    zf = zip_fopen_index(za, i, 0);
		    if (!zf) {
			return 1;
		    }

#if defined(__linux__)
		    fd = open(sb.name, O_RDWR | O_TRUNC | O_CREAT, 0644);
#elif defined (_WIN32)
//			std::cerr << " Open file " << absPath << std::endl;
		    fd = _open(absPath.c_str(), _O_RDWR | _O_TRUNC | _O_CREAT | _O_BINARY, _S_IREAD | _S_IWRITE);
#endif
		    if (fd < 0) {
			//std::cerr << __FILE__ << " " << __LINE__ << " Open failed" << std::endl;
			return 1;
		    }

		    sum = 0;
		    int sizeoffile = 0;
		    while (sum != sb.size) {
			len = zip_fread(zf, buf, 100);
			if (len < 0) {
			    return 1;
			}
#if defined(__linux__)
			sizeoffile = write(fd, buf, len);
#elif defined (_WIN32)
			sizeoffile = _write(fd, buf, len);
#endif
			if (sizeoffile < 0) {
			    //std::cerr << __FILE__ << " " << __LINE__ << " Write failed" << std::endl;
			    return 1;
			}
			sum += len;
		    }
#if defined(__linux__)
		    close(fd);
#elif defined (_WIN32)
		    if (_close(fd) < 0) {
			//std::cerr << __FILE__ << " " << __LINE__ << " Close failed" << std::endl;
			return 1;
		    }
#endif
		    zip_fclose(zf);
		}
	    }
	}

	if (zip_close(za) == -1) {
	    return 1;
	}

	return 0;
}

static void safe_create_dir(const char* dir) {
//    std::cerr << "Folder to create " << dir << std::endl;
#if defined (__linux__)
    int r;
    if ((r = mkdir(dir, S_IRWXU | S_IRWXG | S_IROTH | S_IXOTH)) < 0 && errno != EEXIST )
#elif defined (_WIN32)
    if (_mkdir(dir) < 0 && errno != EEXIST )
#endif
    {
#if defined (_WIN32)
	std::cerr << "Win error " << GetLastError() << std::endl;
#endif
	std::cerr << "Folder " << dir << " already exists." << std::endl;
	boinc_finish(153);
    }
}

void test_local_config(std::string dirPath, std::string filePath) { 
#if defined (__linux__)
    int r = mkdir(dirPath.c_str(), S_IRWXU | S_IRWXG | S_IROTH | S_IXOTH);
    if (r == 0 || errno == EEXIST)
#elif defined (_WIN32)
    if (CreateDirectory(dirPath.c_str(), NULL) || ERROR_ALREADY_EXISTS == GetLastError())
#endif
    {
	  std::ofstream newConfig (filePath);
	  std::cerr << "Making new empty file " << filePath << std::endl;
	  newConfig.flush();
	  newConfig.close();
    } else {
	std::cerr << "Couldn't create folder with local config file." << std::endl;
	boinc_finish(151);
    }
}

void prepare_args(std::vector<char*>& args, std::string& name) {
    std::string *localConfigContent = new std::string;
    std::string *dict1 = new std::string;
    std::string *dict2 = new std::string;
    std::string *hashFile = new std::string;

    *localConfigContent = "";
    std::ifstream localConfig;
    std::string dirPath;
    std::string filePath;

    char* absPathPtr;

#if defined (__linux__)

    char absPath[PATH_MAX];

    absPathPtr = realpath(LOCAL_CONFIG_PATH_LINUX, absPath);
    filePath.assign(absPath);
    filePath += LOCAL_CONFIG_NAME;
    dirPath.assign(absPath);
#elif defined (_WIN32)
    filePath.assign(LOCAL_CONFIG_PATH_WIN);
    filePath += LOCAL_CONFIG_NAME;
    dirPath.assign(LOCAL_CONFIG_PATH_WIN);
#endif

    localConfig.open(filePath);
    if (!localConfig.is_open())
	test_local_config(dirPath, filePath);
    localConfig.open(filePath);
    
    if (localConfig.is_open())
	std::cerr << "Local config is opened" << std::endl;
    else
	std::cerr << "Local config failed to open" << std::endl;

    //std::cerr << "Local.conf char by char" << std::endl;
    for (std::istreambuf_iterator<char> i(localConfig), e; i != e; ++i)
	if (*i != EOF && *i != '\r' && *i != '\n') 
	    *localConfigContent += *i;

    //localConfigContent->assign((std::istreambuf_iterator<char>(localConfig)), std::istreambuf_iterator<char>());
    std::cerr << "file: " << filePath << " localConfigContent: " << *localConfigContent << std::endl;
    //std::cout << "'" << *localConfigContent << "'" << std::endl;
    args.clear();
    //std::cerr << __FILE__ << " " << __LINE__ << std::endl;
    args.push_back(const_cast<char*>(name.c_str()));
    args.push_back(const_cast<char*>("-m"));
    args.push_back(const_cast<char*>(c.taskParams.hash_type.c_str()));
    args.push_back(const_cast<char*>("--force"));
    args.push_back(const_cast<char*>("--machine-readable"));
    //std::cerr << __FILE__ << " " << __LINE__ << std::endl;
    if (*localConfigContent != "")
	args.push_back(const_cast<char*>(localConfigContent->c_str()));
    //std::cerr << __FILE__ << " " << __LINE__ << std::endl;
    if (c.taskParams.mode == "b") {
	args.push_back(const_cast<char*>("-b"));
	return;
    } else {
	//std::cerr << __FILE__ << " " << __LINE__ << std::endl;
	if(!boinc_is_standalone()) {
	    *hashFile = resolve_input_file("data");
	    args.push_back(const_cast<char*>(hashFile->c_str()));
	} else
	    args.push_back(const_cast<char*>("data"));
	if (c.taskParams.attack_mode == "0") {
	    if(!boinc_is_standalone()) {
	    *dict1 = resolve_input_file("dict1");
	    args.push_back(const_cast<char*>(dict1->c_str()));
	    c.taskParams.dictionary1 = true;
	    } else {
		args.push_back(const_cast<char*>("dict1"));
		c.taskParams.dictionary1 = true;
	    }
    //std::cerr << __FILE__ << " " << __LINE__ << std::endl;
	} else if (c.taskParams.attack_mode == "1") {
	    if(!boinc_is_standalone()) {
		*dict1 = resolve_input_file("dict1");
		args.push_back(const_cast<char*>(dict1->c_str()));
		c.taskParams.dictionary1 = true;
		*dict2 = resolve_input_file("dict2");
		args.push_back(const_cast<char*>(dict2->c_str()));
		c.taskParams.dictionary2 = true;
	    } else {
		args.push_back(const_cast<char*>("dict1"));
		c.taskParams.dictionary1 = true;
		args.push_back(const_cast<char*>("dict2"));
		c.taskParams.dictionary2 = true;
	    }
    //std::cerr << __FILE__ << " " << __LINE__ << std::endl;
	} else if (c.taskParams.attack_mode == "3") {
	    args.push_back(const_cast<char*>(c.taskParams.mask.c_str()));
	    //std::cerr << __FILE__ << " " << __LINE__ << std::endl;
	    args.push_back(const_cast<char*>("-s"));
	    args.push_back(const_cast<char*>(c.taskParams.skip.c_str()));
	    if (!c.taskParams.limit.empty()) {
		args.push_back(const_cast<char*>("-l"));
		args.push_back(const_cast<char*>(c.taskParams.limit.c_str()));
	    }
	}
    }
    //std::cerr << __FILE__ << " " << __LINE__ << std::endl;
    args.push_back(const_cast<char*>("-a"));
    args.push_back(const_cast<char*>(c.taskParams.attack_mode.c_str()));
    args.push_back(const_cast<char*>("--status"));
    args.push_back(const_cast<char*>("--status-timer=5"));
    args.push_back(const_cast<char*>("--markov-disable"));
    //args.push_back(const_cast<char*>("--hex-charset"));
    args.push_back(const_cast<char*>("--restore-disable"));
    args.push_back(const_cast<char*>("--potfile-disable"));
    args.push_back(const_cast<char*>("--logfile-disable"));
    args.push_back(const_cast<char*>("--outfile-format=2"));
    args.push_back(const_cast<char*>("--quiet"));
    args.push_back(const_cast<char*>("--gpu-temp-disable"));
    //std::cerr << __FILE__ << " " << __LINE__ << std::endl;

//    args.push_back(NULL);
}

void launch_hashcat () {

    std::chrono::steady_clock::time_point start;
    std::chrono::steady_clock::time_point end;
    std::string outline;
    std::string executable;
    std::string path = LINUX_PATH_TO_HASHCAT;
    std::vector<char*> vArgs;
    std::vector<std::string> status;//  neplati 0 -> STATUS, 1 -> SPEED, 2 -> EXEC_RUNTIME, 3 -> CURKU, 4 -> PROGRESS, 5 -> RECHASH, 6 -> RECSALT, 7 -> TEMP, 8 -> REJECTED
    status.resize(11);
    status[0] = "2";
    status[5] = "0";
    status[6] = "1";

    //std::cerr << __FILE__ << " " << __LINE__ << std::endl;
#if defined (__linux__) //|| __FREEBSD__

    int pid;
    int fd[2];

#if defined(__i386__)
    executable = "hashcat32.bin";
#elif defined(__x86_64__)
    executable = "hashcat64.bin";
#endif

    prepare_args(vArgs, executable);
    if ( !file_exists(path + executable) ) {
	std::cerr << "File " << path + executable << " doesnt exist." << std::endl;
	boinc_finish(6);
    }

    if(pipe(fd) < 0)
	boinc_finish(1);

    pid = fork();

    if(pid == -1) {
	std::cerr << "fork failed\n" << std::endl;
	boinc_finish(-1);
    }

    else if(pid == 0) {//child

	close(fd[0]); //child doesn't read

	for (auto it: vArgs)
	    std::cout << it << " ";
	std::cout << std::endl;

	dup2(fd[1], 1);
	start = std::chrono::steady_clock::now();
	execv((path + executable).c_str(), &vArgs[0]);
    }

    else { //parrent

	close(fd[1]); //parrent doesn't write
	int len = 1;
	char chBuf[1];

	c.exitCode = -1;

	while (waitpid(pid, (int*)&(c.exitCode), WNOHANG) == 0) {
	//while(!(status[5].compare(status[6]) && !status[0].compare("4")) ) {
	    if ((len = read(fd[0], chBuf, 1)) > 0)
		hashcat_parser(chBuf[0], outline, status);
	    c.exitCode = WEXITSTATUS(c.exitCode);
	    //std::cout << "exitCode: " << c.exitCode << std::endl;
	}

	end = std::chrono::steady_clock::now();
	status[2] = std::to_string(std::chrono::duration_cast<std::chrono::microseconds>(end-start).count()/1000000.0);
	c.set_totaltime(status[2]);
	//std::cerr << "Runtime: " << c.get_totaltime() << std::endl;

	c.exitCode = WEXITSTATUS(c.exitCode);
	std::cout << "Hashcat run status " << c.exitCode << std::endl;

	if(pid < 0) {
	    std::cerr << "Can't run hashcat" << std::endl;
	    boinc_finish(5);
	}
    }

#elif defined(_WIN32) || defined(_WIN64)
    HANDLE childOutRead = NULL;
    HANDLE childOutWrite = NULL;
#if defined (_WIN64)
    executable = ".\\hashcat64.exe";
#elif defined (_WIN32)
    executable = ".\\hashcat32.exe";
#endif
    if (!file_exists(executable)){
	//std::cerr << "File " << path + executable << " doesnt exist." << std::endl;
	boinc_finish(6);
    }
    prepare_args(vArgs, executable);
    //std::cerr << __FILE__ << " " << __LINE__ << std::endl;

    std::string arg;
    arg = vArgs[0];
    for (unsigned long i =1; i < vArgs.size(); i++) {
      std::string s(vArgs[i]);
      arg += " " + s;
    }
    std::cerr << arg << std::endl;

    SECURITY_ATTRIBUTES saAttr;
    saAttr.nLength = sizeof(SECURITY_ATTRIBUTES);
    saAttr.bInheritHandle = TRUE;
    saAttr.lpSecurityDescriptor = NULL;

    if (!CreatePipe(&childOutRead, &childOutWrite, &saAttr, 0))
    std::cerr << "StdoutRd CreatePipe " << GetLastError()<< std::endl;
    if (!SetHandleInformation(childOutRead, HANDLE_FLAG_INHERIT, 0))
    std::cerr << "Stdout SetHandleInformation " << GetLastError()<< std::endl;

    STARTUPINFO si;
    PROCESS_INFORMATION pi;

    ZeroMemory( &pi, sizeof(pi) );
    ZeroMemory( &si, sizeof(si) );

    si.cb = sizeof(STARTUPINFO);
    si.hStdError = childOutWrite;
    si.hStdOutput = childOutWrite;
    si.dwFlags |= STARTF_USESTDHANDLES;

    // Start the child process.
    if( !CreateProcess( NULL,   // No module name (use command line)
		(char*)(arg.c_str()),        // Command line
		NULL,           // Process handle not inheritable
		NULL,           // Thread handle not inheritable
		TRUE,          // Set handle inheritance to FALSE
		0,              // No creation flags
		NULL,           // Use parent's environment block
		NULL,           // Use parent's starting directory
		&si,            // Pointer to STARTUPINFO structure
		&pi )           // Pointer to PROCESS_INFORMATION structure
      ) {
        printf( "CreateProcess failed (%lu).\n", GetLastError() );
        boinc_finish(-1);
    }

    start = std::chrono::steady_clock::now();
    char chBuf[1];
    unsigned long len = 1;
    //HANDLE hParentStdOut = GetStdHandle(STD_OUTPUT_HANDLE);

    c.exitCode = STILL_ACTIVE;
    while (c.exitCode == STILL_ACTIVE) {
	if (ReadFile(childOutRead, chBuf, 1, &len, NULL)) {
	    hashcat_parser(chBuf[0], outline, status);
	}
	GetExitCodeProcess(pi.hProcess, &c.exitCode);
    }

    end = std::chrono::steady_clock::now();

    //std::cerr << "Out of loop" << std::endl;

    // Wait until child process exits.
    WaitForSingleObject( pi.hProcess, INFINITE );
    //std::cerr << "Finished waiting" << std::endl;
    GetExitCodeProcess(pi.hProcess, &c.exitCode);
    //std::cerr << "Hashcat exit code " << c.exitCode << std::endl;
    //std::cerr << "Got exit code" << std::endl;

    // Close process and thread handles.
    CloseHandle( pi.hProcess );
    CloseHandle( pi.hThread );

    status[2] = std::to_string(std::chrono::duration_cast<std::chrono::microseconds>(end-start).count()/1000000.0);
    c.set_totaltime(status[2]);
    //std::cerr << "Runtime: " << c.get_totaltime() << std::endl;

#else
    printf("NOT LINUX NOR WINDOWS!\n");
#endif
}
