#include "parser.h"

extern Control c;

void parse_task_params(std::string inputFileName, TTaskParams& taskParams) {
    
    std::string line, code, param;
    std::ifstream inputStream(inputFileName);

    //std::cerr << "parser.cpp" << __LINE__ << std::endl;
    if (!inputStream.is_open()) 
	return;
    std::cerr << "In file start #####################" << std::endl;
    std::cerr << inputStream.rdbuf();
    std::cerr << "In file end #######################" << std::endl;
    inputStream.clear();                 // clear fail and eof bits
    inputStream.seekg(0, std::ios::beg); // back to the start!
    /**
     * @brief   Safely parse input file and save content into the structure.
     */
    while(!safe_get_line(inputStream, line).eof()) {
	if(!parse_message_new_config(line, code, param)) {
	    std::cerr << "Wrong config file format." << std::endl;
	    boinc_finish(10);
	}

        if(code == "attack_mode") {
	    taskParams.attack_mode = param;
	    //std::cerr << "attack: " << taskParams.attack_mode << std::endl;
        } else if(code == "hash_type") {
            taskParams.hash_type = param;
	    //std::cerr << "hash_type: " << taskParams.hash_type << std::endl;
        } else if(code == "mode") {
            taskParams.mode = param;
	    //std::cerr << "mode: " << taskParams.mode << std::endl;
        } else if(code == "start_index") {
            taskParams.skip = param;
	    taskParams.start = std::stoull(param);
	    taskParams.end += taskParams.start;
	    //std::cerr << "skip: " << taskParams.skip << std::endl;
        } else if(code == "hc_keyspace") {
            taskParams.limit = param;
	    taskParams.end += std::stoull(param);
	    //std::cerr << "hc_keyspace: " << taskParams.limit << std::endl;
        } else if(code == "mask_hc_keyspace") {
	    taskParams.end = std::stoull(param);
        } else if(code == "name") {
            taskParams.name = param;
	    //std::cerr << "name: " << taskParams.name << std::endl;
        } else if(code == "mask") {
            taskParams.mask = param;
	    //std::cerr << "mask: " << taskParams.mask << std::endl;
//        } else if(code == "charset1") {
//	    std::ofstream charsetFile;
//	    charsetFile.open(CHARSET_XML_PATH, std::ios::trunc);
//	    charsetFile << r_base64_decode(param);
//	    charsetFile.close(); 
//	    taskParams.charset1 = CHARSET_XML_PATH;
//	    std::cerr << "charset1: " << taskParams.charset1 << std::endl;
//        } else if(code == "charset2") {
//	    std::ofstream charsetFile;
//	    charsetFile.open(CHARSET_XML_PATH, std::ios::trunc);
//	    charsetFile << r_base64_decode(param);
//	    charsetFile.close(); 
//	    taskParams.charset2 = CHARSET_XML_PATH;
//	    std::cerr << "charset2: " << taskParams.charset2 << std::endl;
//        } else if(code == "charset3") {
//	    std::ofstream charsetFile;
//	    charsetFile.open(CHARSET_XML_PATH, std::ios::trunc);
//	    charsetFile << r_base64_decode(param);
//	    charsetFile.close(); 
//	    taskParams.charset3 = CHARSET_XML_PATH;
//	    std::cerr << "charset: " << taskParams.charset3 << std::endl;
//        } else if(code == "charset4") {
//	    std::ofstream charsetFile;
//	    charsetFile.open(CHARSET_XML_PATH, std::ios::trunc);
//	    charsetFile << r_base64_decode(param);
//	    charsetFile.close(); 
//	    taskParams.charset4 = CHARSET_XML_PATH;
//	    std::cerr << "charset4: " << taskParams.charset4 << std::endl;
//        } else if(code == "passlen_min") {
//            taskParams.passlen_min = param;
//	    std::cerr << "passlen_min: " << taskParams.passlen_min << std::endl;
//        } else if(code == "passlen_max") {
//            taskParams.passlen_max = param;
//	    std::cerr << "passlen_max: " << taskParams.passlen_max << std::endl;
//	//    taskParams.length = std::to_string(taskParams.passlen_min) + ":" + std::to_string(taskParams.passlen_max);
        }
    }

    if (taskParams.end == 0)
	taskParams.end++;
}

std::istream& safe_get_line(std::istream& is, std::string& t) {
    t.clear();
    /**
     * @brief	The characters in the stream are read one-by-one using a std::streambuf.
     *		That is faster than reading them one-by-one using the std::istream.
     * 		Code that uses streambuf this way must be guarded by a sentry object.
     * 		The sentry object performs various tasks, such as thread 
     * 		synchronization and updating the stream state.
     */

    std::istream::sentry se(is, true);
    std::streambuf* sb = is.rdbuf();

    for(;;) {
        int c = sb->sbumpc();
        switch (c) {
        case '\n':
            return is;
        case '\r':
            if(sb->sgetc() == '\n')
                sb->sbumpc();
            return is;
        case EOF:
	    /**
	     * @brief	Also handle the case when the last line has no line ending.
	     */
            if(t.empty())
                is.setstate(std::ios::eofbit);
            return is;
        default:
            t += (char)c;
        }
    }
}

bool parse_message_new_config(std::string message, std::string& part1, std::string& part2) {

    part1.clear();
    part2.clear();
    //message = "|||attack|Int|1|1|||";
    if (message.empty())
	return false;
    int len;
    std::string delimeter = "|";
    size_t next = 0, last = 0;
    uint8_t count = 0; 
    while((next = message.find(delimeter, last)) != std::string::npos) {
	if (count == 3)
	    part1 = message.substr(last, next-last);
	else if(count == 5) {
	    len = std::stoi(message.substr(last, next-last));
	    count++;
	    part2 = message.substr(next+1, len);
	    next = next+1+len;
	}
	count++;	
	last = ++next;
    } 
    //std::cout << "part1 " << part1 << " len " << len << " part2 " << part2 << std::endl;
    //std::cout << "count " << count << std::endl;
   
    if (count == 9)
	return true;
    else
	return false;
}

void hashcat_parser (char buf, std::string& outline, std::vector<std::string>& status) {

    //static uint8_t lines = 0;
    uint64_t it = 0, curku = 0;
    double progress = 0.0;
    std::string delimeter = "\t";
    std::vector<std::string> parsed_line;
    int lastit;

    if (buf != '\n' && buf != '\r')
	outline += buf;
    if (buf == '\n') {
	//std::cerr << "Outline: " << outline << std::endl;
	if (c.taskParams.mode == "b") {
	    it = outline.find(":");
	    std::cout << "it: " << it << std::endl;
	    if (it == 1) {
		it = outline.rfind(":") + 1;
		//std::cerr << "Bench_speed: " << outline.substr(it,outline.length()-it) << std::endl;	
		c.avgSpeed += std::stoull(outline.substr(it,outline.length()-it));
		lastit = outline.rfind(":", it-2) + 1;
		//	std::cout << "It: " << it << " lastit: " << lastit << std::endl;
		//std::cerr << "Exec_time: " << outline.substr(lastit,(it-1)-lastit) << std::endl;	
		c.set_totaltime(outline.substr(lastit,(it-1)-lastit));
	    }

	} else {
	    if (outline.length() > 1) {
		if (outline.front() != 'S') {
		    if (c.errorText == "no code")
			c.errorText = "";
		    c.errorText += outline;
		    c.password = outline;
		} else {
		    auto start = 0U;
		    auto end = outline.find(delimeter);
		    //std::cerr << __FILE__ << " " << __LINE__ << std::endl;

		    while (end != std::string::npos) {
			parsed_line.push_back(outline.substr(start, end - start));
			start = end + delimeter.length();
			end = outline.find(delimeter, start);
		    }
		    parsed_line.push_back(outline.substr(start, end));

		    //std::cerr << __FILE__ << " " << __LINE__ << std::endl;
		    if (parsed_line[it++] == "STATUS") {
			if (c.password != "Not found" && parsed_line[it] != "5")
			    c.password = "Not found";
			status[0] = parsed_line[it++];

			//std::cerr << __FILE__ << " " << __LINE__ << std::endl;
			if (parsed_line[it++] == "SPEED") {
			    //std::cerr << __FILE__ << " " << __LINE__ << std::endl;
			    while (parsed_line[it] != "EXEC_RUNTIME") {
	//			b += std::stoi(parsed_line[it++]);
				it+=2; // skip 1\t
			    }
			    //status[1] = std::to_string(b);
			}
			if (parsed_line[it++] == "EXEC_RUNTIME") {
			    //std::cerr << __FILE__ << " " << __LINE__ << std::endl;
			    while (parsed_line[it] != "CURKU") {
				it++;
			    }
			}
			if (parsed_line[it++] == "CURKU"){
			    // CURKU = current keyspace unit
			    //std::cerr << __FILE__ << " " << __LINE__ << std::endl;
			    curku = std::stoull(parsed_line[it]);
			    status[4] = parsed_line[it++];

			    progress = (curku - c.taskParams.start) / (double)(c.taskParams.end - c.taskParams.start); 
			    if (!boinc_is_standalone()) {
				boinc_fraction_done(progress);
				std::cerr << "Progress " << progress << std::endl;
			    } else
				std::cout << "Progress " << progress << std::endl;
			}
			/*
			if (parsed_line[it++] == "PROGRESS") {
			    //std::cerr << __FILE__ << " " << __LINE__ << std::endl;
			    status[5] = parsed_line[it++]; // a
			    //std::cerr << "parsed_line[" << it << "] " << parsed_line[it] << std::endl;
			    status[6] = parsed_line[it++]; // b
			}
			if (parsed_line[it++] == "RECHASH"){
			    
			    //std::cerr << __FILE__ << " " << __LINE__ << std::endl;
			    status[7] = parsed_line[it++];
			}
			if (parsed_line[it++] == "RECSALT"){

			    //std::cerr << __FILE__ << " " << __LINE__ << std::endl;
			    status[8] = parsed_line[it++];
			}
			if (parsed_line[it++] == "TEMP") {
			    b = 0;
			    a = 0;
			    while (parsed_line[it+1] != "REJECTED") {
				//std::cerr << __FILE__ << " " << __LINE__ << std::endl;
			//	sum += std::stoi(parsed_line[it++]);
				b++;
			    }
			    status[9] = std::to_string(a/b);
			}
			if (parsed_line[it++] == "REJECTED"){
			    //std::cerr << __FILE__ << " " << __LINE__ << std::endl;
			    status[10] = parsed_line[it++];
			}
			*/
		    //std::cerr << __FILE__ << " " << __LINE__ << std::endl;

			c.set_totaltime(status[2]);
		    }
		}
	    }
	    std::cerr << "Password: " << c.password << std::endl;
	    //std::cerr << "------------------------------------------------------------------------------------------" << std::endl;
	}

	//std::cout << "Runner: " << outline << std::endl;
	outline.clear();
	//status.clear();
	parsed_line.clear();
    }
}
